@@ -62,6 +62,10 @@ const (
 	BuildStageUploadOnFailureArtifacts BuildStage = "upload_artifacts_on_failure"
 )
 
+const (
+	FFDockerHelperImageV2 string = "FF_DOCKER_HELPER_IMAGE_V2"
+)
+
 type Build struct {
 	JobResponse `yaml:",inline"`
 
@@ -149,6 +153,11 @@ func (b *Build) StartBuild(rootDir, cacheDir string, sharedDir bool) {
 	b.RootDir = rootDir
 	b.BuildDir = path.Join(rootDir, b.ProjectUniqueDir(sharedDir))
 	b.CacheDir = path.Join(cacheDir, b.ProjectUniqueDir(false))
+
+	// invalidate variables cache:
+	// as some variables are based on dynamic
+	// state after build starts
+	b.allVariables = nil
 }
 
 func (b *Build) executeStage(ctx context.Context, buildStage BuildStage, executor Executor) error {
@@ -469,6 +478,7 @@ func (b *Build) Run(globalConfig *Config, trace JobTrace) (err error) {
 	defer cancel()
 
 	trace.SetCancelFunc(cancel)
+	trace.SetMasked(b.GetAllVariables().Masked())
 
 	options := ExecutorPrepareOptions{
 		Config:  b.Runner,
@@ -528,28 +538,45 @@ func (b *Build) GetSharedEnvVariable() JobVariable {
 	return env
 }
 
-func (b *Build) GetCITLSVariables() JobVariables {
+func (b *Build) GetTLSVariables(caFile, certFile, keyFile string) JobVariables {
 	variables := JobVariables{}
+
 	if b.TLSCAChain != "" {
-		variables = append(variables, JobVariable{tls.VariableCAFile, b.TLSCAChain, true, true, true})
+		variables = append(variables, JobVariable{
+			Key:      caFile,
+			Value:    b.TLSCAChain,
+			Public:   true,
+			Internal: true,
+			File:     true,
+		})
 	}
+
 	if b.TLSAuthCert != "" && b.TLSAuthKey != "" {
-		variables = append(variables, JobVariable{tls.VariableCertFile, b.TLSAuthCert, true, true, true})
-		variables = append(variables, JobVariable{tls.VariableKeyFile, b.TLSAuthKey, true, true, true})
+		variables = append(variables, JobVariable{
+			Key:      certFile,
+			Value:    b.TLSAuthCert,
+			Public:   true,
+			Internal: true,
+			File:     true,
+		})
+
+		variables = append(variables, JobVariable{
+			Key:      keyFile,
+			Value:    b.TLSAuthKey,
+			Internal: true,
+			File:     true,
+		})
 	}
+
 	return variables
 }
 
+func (b *Build) GetCITLSVariables() JobVariables {
+	return b.GetTLSVariables(tls.VariableCAFile, tls.VariableCertFile, tls.VariableKeyFile)
+}
+
 func (b *Build) GetGitTLSVariables() JobVariables {
-	variables := JobVariables{}
-	if b.TLSCAChain != "" {
-		variables = append(variables, JobVariable{"GIT_SSL_CAINFO", b.TLSCAChain, true, true, true})
-	}
-	if b.TLSAuthCert != "" && b.TLSAuthKey != "" {
-		variables = append(variables, JobVariable{"GIT_SSL_CERT", b.TLSAuthCert, true, true, true})
-		variables = append(variables, JobVariable{"GIT_SSL_KEY", b.TLSAuthKey, true, true, true})
-	}
-	return variables
+	return b.GetTLSVariables("GIT_SSL_CAINFO", "GIT_SSL_CERT", "GIT_SSL_KEY")
 }
 
 func (b *Build) IsSharedEnv() bool {
@@ -595,6 +622,8 @@ func (b *Build) GetRemoteURL() string {
 	return fmt.Sprintf("%sgitlab-ci-token:%s@%s/%s.git", splits[0], ciJobToken, splits[1], ciProjectPath)
 }
 
+// GetGitDepth is deprecated and will be removed in 12.0, use build.GitInfo.Depth instead
+// TODO: Remove in 12.0
 func (b *Build) GetGitDepth() string {
 	return b.GetAllVariables().Get("GIT_DEPTH")
 }
@@ -706,6 +735,10 @@ func (b *Build) Duration() time.Duration {
 	return time.Since(b.createdAt)
 }
 
+func (b *Build) RefspecsAvailable() bool {
+	return len(b.GitInfo.Refspecs) > 0
+}
+
 func NewBuild(jobData JobResponse, runnerConfig *RunnerConfig, systemInterrupt chan os.Signal, executorData ExecutorData) (*Build, error) {
 	// Attempt to perform a deep copy of the RunnerConfig
 	runnerConfigCopy, err := runnerConfig.DeepCopy()
